# -*- coding: utf-8 -*-

import torch
from torch import nn, Tensor

from torchnorms.tnorms.base import BaseTNorm

from typing import Optional


class FrankTNorm(BaseTNorm):
    def __init__(self,
                 p: Optional[Tensor] = None,
                 default_p: float = 0.5) -> None:
        super().__init__()
        self.p = p
        if self.p is None:
            self.p = nn.Parameter(torch.tensor(default_p))
        assert len(self.p.shape) == 0

        self.relu = torch.nn.ReLU()
        self.eps = 0.001

    def __call__(self,
                 a: Tensor,
                 b: Tensor) -> Tensor:

        res: Optional[Tensor] = None

        self.p.data = self.relu(self.p.data)

        if self.p == 0 or self.p == 1:
            self.p.data += self.eps

        res =  1.0 + ((torch.pow(self.p,a) -1 ) * (torch.pow(self.p,b) - 1 ) / (self.p - 1.0))
        res = torch.log(res)

        #log base change trick -> change base to p
        res = res / torch.log(self.p)

        return res



import torch
from typing import Optional
from torch import nn, Tensor
from torchnorms.negations.base import BaseNegation


class WeberNegation(BaseNegation):
    def __init__(self,
                 p: Optional[Tensor] = None,
                 default_p: float = 0.1) -> None:
        super().__init__()
        self.p = p
        if self.p is None:
            assert default_p > -1
            self.p = nn.Parameter(torch.tensor(default_p))
        assert len(self.p.shape) == 0

        self.relu = torch.nn.ReLU()
        self.eps = 0.01

    def __call__(self,
                 a: Tensor) -> Tensor:

        res: Optional[Tensor] = None

        self.p.data = self.relu(self.p.data) - 1

        if self.p == -1:
            self.p.data += self.eps

        res = (1.0 - a) / (1.0 + self.p * a)

        return res



# -*- coding: utf-8 -*-

from torch import nn, Tensor

from abc import abstractmethod

import logging

logger = logging.getLogger(__name__)


class BaseNegation(nn.Module):
    def __init__(self) -> None:
        super().__init__()

    @abstractmethod
    def __call__(self,
              a: Tensor) -> Tensor:
        raise NotImplementedError



import torchnorms.tnorms.base
import torchnorms.tnorms.aa
import torchnorms.tnorms.additive_generator
import torchnorms.tnorms.classic
import torchnorms.tnorms.dombi
import torchnorms.tnorms.dubois
import torchnorms.tnorms.hamacher
import torchnorms.tnorms.ss
import torchnorms.tnorms.weber
import torchnorms.tnorms.yager
import torchnorms.tnorms.frank



# -*- coding: utf-8 -*-

import torch
from torch import Tensor
from typing import Optional
from torchnorms.negations.base import BaseNegation

class StandardNegation(BaseNegation):
    def __init__(self) -> None:
        super().__init__()

    @classmethod
    def __call__(cls,
                 a: Tensor) -> Tensor:
        return 1.0 - a


class StrictNegation(BaseNegation):
    def __init__(self) -> None:
        super().__init__()

    @classmethod
    def __call__(cls,
                 a: Tensor) -> Tensor:
        return 1.0 - torch.pow(a,2)

class StrictCosNegation(BaseNegation):
    def __init__(self) -> None:
        super().__init__()

    @classmethod
    def __call__(cls,
                 a: Tensor) -> Tensor:

        res:  Optional[Tensor] = None
        pi = torch.acos(torch.zeros(1)).item() * 2
        res = 0.5 * (torch.cos(pi * a))
        return res



import setuptools
import os

with open("README.md", "r") as f:
    long_description = f.read()

setuptools.setup(
    name="torchnorms", #
    version="1.2.3",
    author="Pasquale Minervini, Erik Arakelyan",
    author_email="",
    description="Differentiable Fuzzy Logic operators for ",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pminervini/torch-norms",
    packages=setuptools.find_packages(),
    # package_dir={"": "torchnorms"},
    install_requires=[],
    classifiers=[
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
        "Topic :: Software Development :: Libraries",
    ],
    python_requires=">=3.6",
)
